// Copyright 2022 Krajowa Izba Rozliczeniowa. All rights reserved.

let _nativeConnected = false,
		_debug = false,
		_errorHostNotStarted = "Szafir SDK Chrome Host nie został uruchomiony",
		_resultPartsMaxNo = 0, _resultParts = [];

const _commands = { load: "load", unload: "unload",
		startComponentURI: "startComponentURI", startComponent: "startComponent",
		getTaskListURI: "getTaskListURI", getTaskList: "getTaskList",
		doTaskList: "doTaskList", doTaskListNoWindow: "doTaskListNoWindow", doTaskListNoWindow1Pass: "doTaskListNoWindow1Pass", doTaskListNoWindow2Pass: "doTaskListNoWindow2Pass",
		doTaskListURI: "doTaskListURI", doTaskListNoWindowURI: "doTaskListNoWindowURI", doTaskListNoWindow1PassURI: "doTaskListNoWindow1PassURI", doTaskListNoWindow2PassURI: "doTaskListNoWindow2PassURI",
		isCardInReader: "isCardInReader", getSignatureInfo: "getSignatureInfo", listCertificates: "listCertificates",
		test: "test",
		partAck: "partAck"
	},
	_status = { ok: "OK", error: "ERROR" };

function debugLog(...args) {
	if (_debug === true) {
		let msg = args.shift();
		if (args.length > 0) {
			console.log(msg, args);
		} else {
			console.log(msg);
		}
	}
}

function WND_postMsg(params) {
	const msg = Object.assign({ type: "SZAFIR_WND_MSG" }, params); //, status: _status.ok
	window.postMessage(JSON.stringify(msg), "*");
}

function WND_postErrorMsg(params) {
	const msg = Object.assign({ type: "SZAFIR_WND_MSG", status: _status.error }, params);
	window.postMessage(JSON.stringify(msg), "*");
}

function handleWindowMessages(e) {
	// We only accept messages from ourselves
	if (e.source != window) {
		return;
	}

	const data = _parseMessageData(e.data);
	if (data && data.type == "SZAFIR_EXT_MSG") {
		debugLog("CNT.js : HANDLE WINDOW MESSAGE: ", data);

		switch (data.params.command) {
			case _commands.load:
				HWND_loadSzafir(data.params);
				break;
			case _commands.unload:
				HWND_unloadSzafir();
				break;
			default:
				HWND_SzafirCommand(data.params);
				break;
		}
	}

	function _parseMessageData(data) {
		try {
			return JSON.parse(data);
		} catch(e) {
			return data;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// BEGIN - background.js communication

function handleNativeMessages(jsonResp, sender, callbackFunction) {
	let jsonRespLog = Object.assign({}, jsonResp);
	if ((jsonRespLog.result ?? '').length > 100) {
		jsonRespLog.result = jsonRespLog.result.slice(0,50) + `...[truncated:${jsonRespLog.result.length - 100}]...` + jsonRespLog.result.slice(-50);
	}
	debugLog("CNT.js : HANDLE NATIVE MESSAGE: ", jsonRespLog);

	switch (jsonResp.command) {
		case _commands.load:
			HNTV_onLoad(jsonResp);
			break;
		case _commands.unload:
			HNTV_onUnload(jsonResp);
			break;
		case "switchConsole":
			chrome.runtime.sendMessage({command: "switchConsole", text: "Przełącz konsolę Szafir SDK!!!"});
			callbackFunction({enabled: "enabled"});
			break;
		default:
			HNTV_onSzafirCommand(jsonResp);
			break;
	}
}

function HNTV_onLoad(jsonResp) {
	_nativeConnected = (jsonResp.status === _status.ok);
	if (_nativeConnected) {
		jsonResp.result = "LOADED";
	} else {
		jsonResp.status = _status.ok;
		jsonResp.result = (jsonResp.result === "NOHOSTAPP" ? "NOHOSTAPP" : "ERROR");
		// komunikat zawiera też current_version i required_version
	}
	WND_postMsg(jsonResp);
}

function HNTV_onUnload(jsonResp) {
	_nativeConnected = false;
	jsonResp.result = "UNLOADED";
	WND_postMsg(jsonResp);
}

function HNTV_onSzafirCommand(jsonResp) {
	if (jsonResp.command == _commands.isCardInReader) {
		jsonResp.result = (jsonResp.result === "true");
	}
	if (jsonResp.part === undefined) {
		WND_postMsg(jsonResp);

	} else {
		_resultPartsMaxNo = Math.max(_resultPartsMaxNo, jsonResp.part + 1)
		debugLog(`CNT.js : MULTIPART RESULT PART [${_resultPartsMaxNo - jsonResp.part}/${_resultPartsMaxNo}]`);
		_resultParts.push(jsonResp.result);
		if (jsonResp.part <= 0) {
			jsonResp.result = _resultParts.join('');
			_resultParts.length = 0;
			_resultPartsMaxNo = 0;
			WND_postMsg(jsonResp);
		} else {
			HWND_SzafirCommand({command: _commands.partAck});
		}
	}
}

// END = background.js communication
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// BEGIN - SDK functions

function HWND_loadSzafir(params) {
	_debug = params.config.debug || false;
	debugLog("CNT.js : HWND_loadSzafir");
	const paramsJson = Object.assign({ command: _commands.load }, params.config);
	chrome.runtime.sendMessage(paramsJson);
}

function HWND_unloadSzafir() {
	debugLog("CNT.js : HWND_unloadSzafir");
	if (_nativeConnected) {
		chrome.runtime.sendMessage({command: _commands.unload});
		handleNativeMessages({command: _commands.unload});
	} else {
		WND_postErrorMsg({command: _commands.unload, result: _errorHostNotStarted});
	}
}

function HWND_SzafirCommand(params) {
	debugLog("CNT.js : HWND_SzafirCommand: ", params);
	if (_nativeConnected) {
		try {
			let callback = (resp) => console.debug("RESP:", resp);
			chrome.runtime.sendMessage(params, callback);
		} catch(err) {
			debugLog("CNT.js : HWND_SzafirCommand error: ", err);
			WND_postErrorMsg({command: params.command, result: `${err.name}: ${err.message}`});
		}
	} else {
		WND_postErrorMsg({command: params.command, result: _errorHostNotStarted});
	}
}

// END - SDK functions
////////////////////////////////////////////////////////////////////////////////////////////////////

function init() {
	if (document.body.dataset.szafirsdkReady) {
		console.log("CNT.js : Init Szafir SDK Web Module Extension");

		chrome.runtime.onMessage.addListener(handleNativeMessages);
		window.addEventListener("message", handleWindowMessages);

		document.body.dataset.szafirsdkReady = "EXT_INITIALIZED";
	}
}

//document.addEventListener("DOMContentLoaded", (event) => init());
//window.addEventListener("load", (event) => init());

init();
